;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x4d68ddaaf1247b70a43690c6e717352c . "./code.cg2js/cg-status-bar.fasl")
(#x79183594adb5bc45564633c8ebe1b57d . "./code.cg2js/cg-hotspot.fasl")
(#x43c1dbed7a182ee195ebb6c3e1fc1026 . "./code.cg2js/cg-toolbar.fasl")
(#xde27370d49b1224e830acf3fbedf385f . "./code.cg2js/cg-clipboard-stack.fasl")
(#x1e76d1c77a6b64bc8ca4e5cb1a0304bb . "./code.cg2js/cg-tooltip.fasl")
(#x3ce4029e167efa66255f4dbec4936216 . "./code.cg2js/cg-calendar.fasl")
(#x3b0d979fcebc24c30785c1b2457bf675 . "./code.cg2js/cg-item-list.fasl")
(#xbd15f138dd5bc4edc5bc1518c1876e96 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc3a3a7905537cc73cd712e69576b507e . "./code.cg2js/cg-trackbar.fasl")
(#xa04f558d4f11991355d74da57b10aed1 . "./code.cg2js/cg-class-grid.fasl")
(#xc20e8f14e88d03c516c166044c9926bc . "./code.cg2js/cg-dialog-item.fasl")
(#x91872c9a33cf0b53ee36fb22e6d5b302 . "./code.cg2js/cg-comtab.fasl")
(#x9eaf27e64a8f06401df90aa3464e798f . "./code.cg2js/cg-lisp-text.fasl")
(#x29449061e56707a01d4e1a8a5b0f3d37 . "./code.cg2js/cg-button.fasl")
(#x180a886a8a6db08e48e5bbdb0db107b8 . "./code.cg2js/cg-printing.fasl")
(#xcad2a00b0295475c59b0d080444d24e9 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x35a41340175b713a3de9a46b25189521 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xddeab2ac013f19e720f8927de347c297 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x0eadd84d836805a55709523f6792325e . "./code.cg2js/cg-selected-object.fasl")
(#x135668cc805db4ae10f7d8bd91b15cb9 . "./code.cg2js/cg-text-or-combo.fasl")
(#x12e1c7469d581a2453716f0cebcb6cd4 . "./code.cg2js/cg-graphics-context.fasl")
(#x238c3fd2e63504881b4fda94e0831618 . "./code.cg2js/cg-edit-in-place.fasl")
(#x0c5639d8e1fb898b961afafb7a915681 . "./code.cg2js/cg-string-dialog.fasl")
(#xc95b2b911b818b7c8b55248399f112ed . "./code.cg2js/cg-list-view.fasl")
(#xb78b91f8650f67730c0b152360e20d0b . "./code.cg2js/cg-intersect.fasl")
(#x4730e105fe828b9bbd069c53777eca64 . "./code.cg2js/cg-text-widget.fasl")
(#xc7f075ce14ee1eab463102f6cb2e66a7 . "./code.cg2js/cg-directory-dialog.fasl")
(#xfaac4eb3447072237bcc6145ee5d0395 . "./code.cg2js/cg-file-dialog.fasl")
(#x3502710f163191dacd4002d33d7f101c . "./code.cg2js/cg-dde-utils.fasl")
(#x5d478957d5a2eba3f6f6090ecda9468f . "./code.cg2js/cg-nodes.fasl")
(#x4560569fe20b5f104c5159d930f091e4 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xea7778176e8c38f52d013175a6d6530c . "./code.cg2js/cg-up-down-control.fasl")
(#x1347a3339105868e079f3948210808f2 . "./code.cg2js/cg-get-position.fasl")
(#x0aed83746fba2d2b0bc2d2d9fb382192 . "./code.cg2js/cg-toggling-widget.fasl")
(#xe528f3d2a5a1e1d9db36e6d844d5f7ae . "./code.cg2js/cg-acache.fasl")
(#x05897e6a9bbd106d5010be63892cd5ff . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x72120d7f406fe01a6743a45662e0ee98 . "./code.cg2js/cg-base.fasl")
(#xf2b759002ac3df33b3b1951a600ee890 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x989289ff8b603edd3906600a362fa2ea . "./code.cg2js/cg-find-string-dialog.fasl")
(#x7c4710c4373a4f19bf55aa0674cefa7f . "./code.cg2js/cg-property.fasl")
(#x2c6730c6b6763df6508c1f04c9ea4dc3 . "./code.cg2js/cg-editable-text.fasl")
(#x2973065ec498c94e3506fb7217edbe49 . "./code.cg2js/cg-lamp.fasl")
(#x1f3f71fb2a141249c38b96b56320a5a4 . "./code.cg2js/cg-progress-indicator.fasl")
(#x466e98b48a80041add74289a4025bfe0 . "./code.cg2js/cg-chart-widget.fasl")
(#x51f9cd74aadc37cea40e84d07f413e59 . "./code.cg2js/cg-common-control.fasl")
(#x855e06eb7581804c72fb0fb17758ebde . "./code.cg2js/cg-font-dialog.fasl")
(#x0da8e0716273b12a525b5cbdd11df80c . "./code.cg2js/cg-gesture-emulation.fasl")
(#x0fd4ced912fff1891adcc2f6cb506573 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xf9f31dd2e3dfc03b861cb930c5d8cc62 . "./code.cg2js/cg-os-window.fasl")
(#x4268ca8fb0a3a79437ed6ba1ecc5d1f1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7db306670e00a10c24db7d2c91753dc8 . "./code.cg2js/cg-pixmap.fasl")
(#xc35562a53665832e2e3ec6d67a5ab15a . "./code.cg2js/cg-chart-or-plot.fasl")
(#x41ecc40617327622de0055de35b2a8a4 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3f289b5d298bc757bd2ee4890e3ea38f . "./code.cg2js/cg-scaling-stream.fasl")
(#x9dde44075bf3ab0e2752b63dc906c4c6 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x29d9aacbb3934129506d66e031068121 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0d8ab2ae1569673947f1a9838fe6ffe6 . "./code.cg2js/cg-timer.fasl")
(#x3e328438d0a75c72066450af0d59b7f1 . "./code.cg2js/cg-scroll-bar.fasl")
(#x84c71ab87863c10901a9ffeb23a18a49 . "./code.cg2js/cg-caret.fasl")
(#xe67a17ee04c68681669cd0065199a0b0 . "./code.cg2js/cg-template-string.fasl")
(#x3d9db395e3d4b209aef19aa8e4760ff1 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x3c36ca074f3bdd0c6361eec78e24978d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x57987bd948d9e3c81b1bca7df93e26c2 . "./code.cg2js/cg-combo-box.fasl")
(#xf0adec768432608f960667ad29cebaf1 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x73ec35f35f5fc5def0402b1ae21736f1 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xabdd00f7153d0aed6f39970b91a1d5cb . "./code.cg2js/cg-menu-tooltip.fasl")
(#x29afc024ed2842e8a27aaa2a7b694c91 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xc927b2982de4ed7abda8ce85abef5992 . "./code.cg2js/cg-player.fasl")
(#xb7aa43057b8b92079a4ec73c4c423c21 . "./code.cg2js/cg-outline.fasl")
(#xe26a3c83ddc4789b90f6d93bc4b35ea7 . "./code.cg2js/cg-class-support.fasl")
(#xb425cb7a77448f0300e78ed3dcc10ec6 . "./code.cg2js/cg-lettered-menu.fasl")
(#x06a6feea205f541418f0b0beb83c83b1 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb87f0ad77ff1f3507a5b152841da9511 . "./code.cg2js/cg-fill-texture.fasl")
(#xcff1c3a5cc905d27ec86af2862826db4 . "./code.cg2js/cg-lisp-widget.fasl")
(#xf71af9a1ca79be447218c3eb9ad3d23f . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xae0823b44d36d2e7e413b8a8403db4e3 . "./code.cg2js/cg-clipboard.fasl")
(#xa7f4ebe1a3a204ea5976c90c3002fd09 . "./code.cg2js/cg-wrap-string.fasl")
(#xc2fab9398f6435cb1b374de4cc115423 . "./code.cg2js/cg-os-widget.fasl")
(#x8d55560c025bfbc9d7574570751e553a . "./code.cg2js/cg-check-box.fasl")
(#xf2ed8ca58ffc7db6221f63bf7c1c8e39 . "./code.cg2js/cg-choice-list.fasl")
(#xa8c8ad4faff8a726cb1fa8c12fecda90 . "./code.cg2js/cg-layout.fasl")
(#x3f415342c463f09ebac4e215643f1a7a . "./code.cg2js/cg-group-box.fasl")
(#x89404994f25f7174cbf540fba89885ae . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x094f907791ebaceeffb8ba2d2ba87b69 . "./code.cg2js/cg-radio-button.fasl")
(#x5b7cf26e34413f21eac3497d835b7d21 . "./code.cg2js/cg-tab-control.fasl")
(#x73a0030b10194de829c39b11ed5ca717 . "./code.cg2js/cg.fasl")
(#xf04bd04300b2107170323ee08218c57b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xb68c8a4b338e271c51a93ec8659c0e98 . "./code.cg2js/cg-dropping-outline.fasl")
(#x5aceaea83be5e6083a0db80aecedf726 . "./code.cg2js/cg-get-pixmap.fasl")
(#xbf2ccb5f0db7ee1b40edfe72b1dd18f1 . "./code.cg2js/cg-grid-widget.fasl")
(#xc6f2af10af97f76dee34f6b8e166aa77 . "./code.cg2js/cg-utility-dialog.fasl")
(#x76a79c7b6dadd69031fa155989056ebf . "./code.cg2js/cg-static-text.fasl")
(#xaa91ea2d32f25b9523956494454f32a7 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xda6a1c39005a9c436a9723d429f50283 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xe7e61d933c19ef036ec9483c41f2a342 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x6cff97609868df9c529edce67fa78c07 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9fba17fc3b4038349414a8f81ae34cd5 . "./code.cg2js/cg-plot-widget.fasl")
(#xf0a3dc73a59c7ad4d0e737b1af2d45a3 . "./code.cg2js/cg-color-dialog.fasl")
(#x46c7f04f0180043f7faa56c47d1e65a1 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x2a98304f013f24a843ce48633feb2411 . "./code.cg2js/cg-icon.fasl")
(#xf695c98d5db4aada7049acbf6b37664e . "./code.cg2js/cg-dev.fasl")
(#x8b4e656427153c162753f8b77918a9a6 . "./code.cg2js/cg-choose-printer.fasl")
(#x11f92f57c862f7e5c46f70bd6ae18574 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xb5e0f8c68a147f03f4444db046789b6c . "./code.cg2js/cg-menu.fasl")
(#xe1cb6d97f6890e19f6d59cd3c6e70a95 . "./code.cg2js/cg-yes-no-list.fasl")
(#xf5f504d55da23d8e07076d8f6bffa1e3 . "./code.cg2js/cg-palette.fasl")
(#x59f03e654390e1f6e4dc5c95eaee59c9 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x2da4eaa041985a6a4ae706d3e71bc389 . "./code.cg2js/cg-message-dialog.fasl")
(#xed5364da78648114696abbaa8c11e83d . "./code.cg2js/cg-shortcut-menu.fasl")
(#x2f80fd21c5420ab4d47e30f97acd59b1 . "./code.cg2js/cg-split-bar.fasl")
(#xfb42bcb99473cf1820a4a469852e2e42 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x592794018ccd0c7ea20bb50166ae765a . "./code.cg2js/cg-web-browser.fasl")
(#x2600d774533349d5339785b83136acbb . "./code.cg2js/cg-project-window.fasl")
(#x44b6af94424c2df0b529dc9317041eb0 . "./code.cg2js/cg-picture-widget.fasl")
(#x404be4707b668d16a903b86c72bbb14b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x0a189372ee6c73bb8ec6cbc577856498 . "./code.cg2js/cg-object-editor.fasl")
(#x8ad420325bedd739848f0033c01d45d2 . "./code.cg2js/cg-paren-matching.fasl")
(#xdde16fc3b6b02c4180cffe7dcb3074b3 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x61a8ba1974ced958aaecc3b4843bf756 . "./code.cg2js/cg-drawable.fasl")
(#x1229b170203b8004223b8ed69fbbb204 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xe246cbdf0348559f4954b3ef8f38b0f2 . "./code.cg2js/cg-html-widget.fasl")
(#x2ee77a65448434d055be6fdeff7b4e6d . "./code.cg2js/cg-curve.fasl")
(#xd8d10789535f2036b930d50ba05fb55b . "./code.cg2js/cg-yes-no-string.fasl")
(#x3ae884ee4949efd8d2b584d3fb844a49 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x0ccb473baa79a9635ca1383f6fe37578 . "./code.cg2js/cg-html-dialog.fasl")
(#xb14477bb6ac2b83359b4692c5cb74e95 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xc32eda93fe7c19f6423e9dd8def9cec5 . "./code.cg2js/cg-tray.fasl")
(#x79de9786afc36e1c917888b316fa4db2 . "./code.idejs/ide-apropos.fasl")
(#xa293c2f8d9be92ba046a32ab03e5ff8b . "./code.idejs/ide-window-list.fasl")
(#xfbcb6f3a5bb5792c34ae553f0ed7d0dc . "./code.idejs/ide-class-browser.fasl")
(#x732b9837642a208077b238d93eae6032 . "./code.idejs/ide-base.fasl")
(#x0befcf326e363a533ea170f49f1a9e17 . "./code.idejs/ide-process-browser.fasl")
(#x8d161c313dff4cf8f4b0e60fcbdb0c1e . "./code.idejs/ide-acache.fasl")
(#x81f93c7a70c9f21e162c57b0fc316814 . "./code.idejs/ide-navigator.fasl")
(#xb27ed5eaab9d95ea3be6ce4de56a97ef . "./code.idejs/ide-trace.fasl")
(#x0c004614c5add5d0ecbabdcece01e2c4 . "./code.idejs/ide-project.fasl")
(#xc08ea4b9c6f7ca18eb410035b6b6b045 . "./code.idejs/ide-patch-dialog.fasl")
(#xf508b5433a4e60ccfbf577f0192636ab . "./code.idejs/ide-grapher.fasl")
(#xd3fba81491da5399be13a120f6f6ff6c . "./code.idejs/ide-dev.fasl")
(#xcc2c1f048f19d72251daf9bde9f010c1 . "./code.idejs/ide-keyshort.fasl")
(#xf76e9d3c29ef4e97e37f0a6d22931768 . "./code.idejs/ide-editor.fasl")
(#x22dbb6e8bc38505620e8aac1fa38d141 . "./code.idejs/ide-inspector.fasl")
(#x28cc7de298edbb4277628ba43989df41 . "./code.idejs/ide.fasl")
(#x09f82cd6a5ee31550cd8194fb761ade4 . "./code.idejs/ide-clipboard.fasl")
(#x2237d94c7bdc8903aebe6e529496ad3a . "./code.idejs/ide-find-definitions.fasl")
(#xb5200661fd72c0ed3b2d8d0091e11a09 . "./code.idejs/ide-form.fasl")
(#x68a7ccba3d9e2efe144cbca16fb853fb . "./code.idejs/ide-intro.fasl")
(#xd8e8037d84a5bf216fadbbd54f70b4b2 . "./code.idejs/ide-find-in-files.fasl")
(#xa771a39a45ca110ae88ec0342fa4a733 . "./code.idejs/ide-package-list.fasl")
(#x878ed33187482f3924cae8c04ee40dd2 . "./code.idejs/ide-class-grapher.fasl")
(#x4469068b47e063cb9811630bd05af4c2 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
